import sys
import subprocess

from langswi.mapping import cyrillic_latin_mapping, cyrillic_symbols


def get_mapping(input: str) -> dict:
    input = ''.join(
        symbol for symbol in input.strip().replace(' ', '') if symbol.isalpha()
    )

    if set(input.lower()) <= cyrillic_symbols:
        mapping = {v: k for k, v in cyrillic_latin_mapping.items()}
    else:
        mapping = cyrillic_latin_mapping

    return mapping


def translate(input: str) -> str:
    mapping = get_mapping(input)

    result = ''
    for symbol in input:
        if ord(symbol) in mapping.keys():
            result += chr(mapping[ord(symbol)])
        else:
            result += symbol

    return result


def get_stdout() -> str:
    args = ('pbpaste', 'r',)
    process = subprocess.Popen(args, stdout=subprocess.PIPE, close_fds=True)
    stdout, stderr = process.communicate()
    return stdout.decode('utf-8')


def main() -> None:
    input = get_stdout()
    result = translate(input)
    sys.stdout.write(result + '\n')



#!/usr/bin/env python

import os.path
from setuptools import setup


def read(name):
    return open(os.path.join(os.path.dirname(__file__), name)).read()


version = [line for line in read('langswi/__init__.py').split('\n') if '__VERSION__' in line][0]

exec(version)


setupconf = dict(
    name='langswi',
    version='.'.join(str(ver) for ver in __VERSION__),
    license='BSD',
    url='https://github.com/sonerayberk/langswi/',
    author='sonerayberk',
    author_email='soner.ayberk@gmail.com',
    description='Cyrillic-latin and reverse text transformation',
    long_description=read('README.md'),
    keywords='language switcher text transformation translation',
    packages=['langswi'],
    entry_points={
        'console_scripts': [
            'langswi=langswi.main:main'
        ]
    },
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
    ],
)

if __name__ == '__main__':
    setup(**setupconf)



# Second Python's rule: Explicit is better than implicit.

alpha_mapping = {
    ord('q'): ord('й'),
    ord('w'): ord('ц'),
    ord('e'): ord('у'),
    ord('r'): ord('к'),
    ord('t'): ord('е'),
    ord('y'): ord('н'),
    ord('u'): ord('г'),
    ord('i'): ord('ш'),
    ord('o'): ord('щ'),
    ord('p'): ord('з'),
    ord('a'): ord('ф'),
    ord('s'): ord('ы'),
    ord('d'): ord('в'),
    ord('f'): ord('а'),
    ord('g'): ord('п'),
    ord('h'): ord('р'),
    ord('j'): ord('о'),
    ord('k'): ord('л'),
    ord('l'): ord('д'),
    ord('z'): ord('я'),
    ord('x'): ord('ч'),
    ord('c'): ord('с'),
    ord('v'): ord('м'),
    ord('b'): ord('и'),
    ord('n'): ord('т'),
    ord('m'): ord('ь'),
}

non_alpha_mapping = {
    ord(';'): ord('ж'),
    ord("'"): ord('э'),
    ord('['): ord('х'),
    ord(']'): ord('ъ'),
    ord(','): ord('б'),
    ord('.'): ord('ю'),
    ord('§'): ord('ё'),
    ord('?'): ord(','),
    ord('&'): ord('?'),
    ord('<'): ord('Б'),
    ord('>'): ord('Ю'),
}

uppers_mapping = {ord(chr(k).upper()): ord(chr(v).upper()) for k, v in alpha_mapping.items()}

cyrillic_latin_mapping = {
    **alpha_mapping,
    **non_alpha_mapping,
    **uppers_mapping
}

cyrillic_symbols = {
    chr(symbol) for symbol in cyrillic_latin_mapping.values() if chr(symbol).isalpha()
}
